﻿using System.Collections.ObjectModel;
using System.ComponentModel;

namespace Grid
{
    public class DataUtil
    {
        #region private fields

        private static ObservableCollection<Supervisor> supervisors;
        private static ObservableCollection<Supervisee> supervisees;
        private static ObservableCollection<Supervisor> all;

        #endregion

        #region private methods

        private static ObservableCollection<Supervisor> CreateSupervisors()
        {
            return new ObservableCollection<Supervisor>
            {
                new Supervisor 
                {
                    EmployeeID = 1, 
                    EmployeeName = "Philip", 
                    Department = "Support",
                    Designation = "Supervisor"
                },

                new Supervisor 
                {
                    EmployeeID = 2, 
                    EmployeeName = "Vladimir", 
                    Department = "Engineering",
                    Designation = "Test manager"
                },
                
                new Supervisor
                {
                    EmployeeID = 3,
                    EmployeeName = "Michael",
                    Department = "Human Resources",
                    Designation = "Senior HR manager"
                }
            };
        }

        private static ObservableCollection<Supervisee> CreateSupervisees()
        {
            return new ObservableCollection<Supervisee>
            {
                new Supervisee
                {
                    EmployeeID = 1,
                    EmployeeName = "Matt",
                    Department = "Support",
                    Designation = "Developer support intern"
                },

                new Supervisee
                {
                    EmployeeID = 2,
                    EmployeeName = "Chang",
                    Department = "Engineering",
                    Designation = "Software engineer in test"
                },     

                new Supervisee
                {
                    EmployeeID = 3,
                    EmployeeName = "Hong",
                    Department = "Engineering",
                    Designation = "Software engineer"
                },

                new Supervisee
                {
                    EmployeeID = 4,
                    EmployeeName = "Ming",
                    Department = "Engineering",
                    Designation = "Senior software engineer"
                },

                new Supervisee
                {
                    EmployeeID = 5,
                    EmployeeName = "Martin",
                    Department = "Engineering",
                    Designation = "Software architect"
                },

                new Supervisee
                {
                    EmployeeID = 6,
                    EmployeeName = "Howard",
                    Department = "Support",
                    Designation = "Developer support engineer"
                },
                new Supervisee
                {
                    EmployeeID = 7,
                    EmployeeName = "Allen",
                    Department = "Support",
                    Designation = "Senior developer support engineer"
                },

                new Supervisee
                {
                    EmployeeID = 8,
                    EmployeeName = "King",
                    Department = "Human Resources",
                    Designation = "HR manager"
                },
                new Supervisee
                {
                    EmployeeID = 9,
                    EmployeeName = "Han",
                    Department = "Engineering",
                    Designation = "Release engineer"
                },
                new Supervisee
                {
                    EmployeeID = 10,
                    EmployeeName = "Smith",
                    Department = "Human Resources",
                    Designation = "Technical requiter"
                },

                new Supervisee
                {
                    EmployeeID = 11,
                    EmployeeName = "Paul",
                    Department = "Engineering",
                    Designation = "Team leader"
                }
            };
        }

        private static ObservableCollection<Supervisor> CreateSupervisorsAndSupervisees()
        {
            ObservableCollection<Supervisor> retVal = CreateSupervisors();

            ObservableCollection<Supervisee> tempSupervisees = CreateSupervisees();

            for (int i = 0; i < retVal.Count; i++)
            {
                for (int j = 0; j < tempSupervisees.Count; j++)
                {
                    if (retVal[i].Department == tempSupervisees[j].Department)
                    {
                        retVal[i].Supervisees.Add(tempSupervisees[j]);
                    }
                }
            }

            return retVal;
        }

        #endregion

        public static ObservableCollection<Supervisor> Supervisors
        {
            get
            {
                if (supervisors == null)
                    supervisors = CreateSupervisors();

                return supervisors;
            }
        }

        public static ObservableCollection<Supervisee> Supervisees
        {
            get
            {
                if (supervisees == null)
                    supervisees = CreateSupervisees();

                return supervisees;
            }
        }

        public static ObservableCollection<Supervisor> All
        {
            get
            {
                if (all == null)
                    all = CreateSupervisorsAndSupervisees();

                return all;
            }
        }
    }

    public class Supervisor : Supervisee, INotifyPropertyChanged
    {
        private ObservableCollection<Supervisee> supervisees;

        public int SupverviseesCount
        {
            get
            {
                return this.Supervisees.Count;
            }
        }

        public ObservableCollection<Supervisee> Supervisees
        {
            get
            {
                if (supervisees == null)
                {
                    supervisees = new ObservableCollection<Supervisee>();
                }

                return this.supervisees;
            }
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion
    }

    public class Supervisee : INotifyPropertyChanged
    {

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion

        #region private members

        private int employeeId;
        private string employeeName;
        private string department;
        private string designation;

        #endregion

        #region public properties
        public int EmployeeID
        {
            get
            {
                return this.employeeId;
            }
            set
            {
                if (this.employeeId != value)
                {
                    this.employeeId = value;
                    OnPropertyChanged("EmployeeID");
                }
            }
        }

        public string EmployeeName
        {
            get
            {
                return this.employeeName;
            }
            set
            {
                if (this.employeeName != value)
                {
                    this.employeeName = value;
                    OnPropertyChanged("EmployeeName");
                }
            }
        }

        public string Department
        {
            get
            {
                return this.department;
            }
            set
            {
                if (this.department != value)
                {
                    this.department = value;
                    OnPropertyChanged("Department");
                }
            }
        }

        public string Designation
        {
            get
            {
                return this.designation;
            }
            set
            {
                if (this.designation != value)
                {
                    this.designation = value;
                    OnPropertyChanged("Designation");
                }
            }
        }

        #endregion
    }
}
